package it.dsestili.jhash.core;

/*
JHash a simple hash code generator
Copyright (C) 2013-2014 Davide Sestili

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

import java.io.File;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;

public class DirectoryScanner 
{
	private List<IScanProgressListener> listeners = new ArrayList<IScanProgressListener>();
	private List<File> files = new LinkedList<File>();
	private final File directory;
	private final boolean recursively;
	private volatile boolean interrupt = false;	
	
	public DirectoryScanner(File directory, boolean recursively) throws Throwable
	{
		if(directory.isFile())
		{
			throw new IllegalArgumentException();
		}

		this.directory = directory;
		this.recursively = recursively;
	}

	public void addIScanProgressListener(IScanProgressListener listener)
	{
		listeners.add(listener);
	}

	private void generateEvents(int pCent, long current, long tot)
	{
		final ScanProgressEvent event = new ScanProgressEvent(this, pCent, current, tot);
		
		for(IScanProgressListener listener : listeners)
		{
			listener.scanProgressEvent(event);
		}
	}

	private void generateEvents()
	{
		final StartScanProgressEvent event = new StartScanProgressEvent(this);
		
		for(IScanProgressListener listener : listeners)
		{
			listener.scanProgressEvent(event);
		}
	}

	private void scan(File directory, boolean isParent) throws Throwable
	{
		if(isParent)
		{
			generateEvents();
		}
		
		File[] sub = directory.listFiles();
		if(sub != null)
		{
			if(isParent)
			{
				generateEvents(0, 0, sub.length);
			}
			
			for(int i = 0; i < sub.length; i++)
			{
				File content = sub[i];
				
				if(interrupt)
				{
					throw new InterruptedException();
				}

				if(content.isDirectory() && !recursively)
				{
					continue;
				}
				
				if(content.isDirectory())
				{
					scan(content, false);
				}
				else
				{
					files.add(content);
				}
				
				if(isParent)
				{
					final int pCent = ((i + 1) * 100) / sub.length;
					generateEvents(pCent, (i + 1), sub.length);
				}
			}
		}
	}

	public void interrupt()
	{
		interrupt = true;
	}

	public File[] getFiles() throws Throwable
	{
		scan(directory, true);
		
		return files.toArray(new File[0]);
	}
	
}
