package it.dsestili.jhash.core;

/*
JHash a simple hash code generator
Copyright (C) 2013-2014 Davide Sestili

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

import it.dsestili.jhash.gui.HashListWindow;
import it.dsestili.jhash.gui.InputPanel;
import it.dsestili.jhash.gui.StatusPanel;

import java.awt.Frame;
import java.io.File;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.SwingUtilities;

public class DirectoryWorkerThread extends WorkerThread implements IScanProgressListener
{
	private volatile File currentFile;
	private volatile int currentIndex, totFiles;
	private final Frame frame;
	private DirectoryScanner scanner = null;
	
	public DirectoryWorkerThread(Frame frame, File file, String algorithm, StatusPanel statusPanel, InputPanel inputPanel, JButton btnStart)
	{
		super(file, algorithm, statusPanel, inputPanel, btnStart);
		
		this.frame = frame;
	}

	@Override
	public void run()
	{
		try
		{
			showHash("", true);
			manageGUI(false);
			
			boolean recursively = getRecursively();

			long start = System.currentTimeMillis();
			
			scanner = new DirectoryScanner(file, recursively);
			scanner.addIScanProgressListener(this);
			File[] files = scanner.getFiles();
			
			if(files.length == 0)
			{
				showStatus("Empty folder");
				JOptionPane.showMessageDialog(frame, "Empty folder!", "Empty folder", JOptionPane.INFORMATION_MESSAGE);
				return;
			}
			
			Vector<Vector<String>> hashList = new Vector<Vector<String>>();
			
			totFiles = files.length;
			
			for(int i = 0; i < files.length; i++)
			{
				currentIndex = i;
				currentFile = files[i];
				
				core = new Core(currentFile, algorithm);
				core.addIProgressListener(this);

				String hash = core.generateHash();
				
				Vector<String> info = new Vector<String>();
				info.add(recursively ? currentFile.getAbsolutePath() : currentFile.getName());
				info.add(hash);
				
				hashList.add(info);
			}

			long elapsed = System.currentTimeMillis() - start;
			JOptionPane.showMessageDialog(null, "Elapsed time: " + Utils.getElapsedTime(elapsed), "Elapsed time", JOptionPane.INFORMATION_MESSAGE);

			showHash(hashList);
		}
		catch(InterruptedException e)
		{
			showStatus("Operation canceled");
		}
		catch(Throwable t)
		{
			showStatus("Error occurred");
			
			t.printStackTrace();
		}
		finally
		{
			manageGUI(true);
		}
	}

	@Override
	public void interruptOperation()
	{
		if(scanner != null)
		{
			scanner.interrupt();
		}
		
		super.interruptOperation();
	}
	
	private boolean getRecursively()
	{
		boolean recursively;
		
		int result = JOptionPane.showConfirmDialog(null, "Scan directories recursively?", "Recursively", JOptionPane.YES_NO_OPTION);
		if(result == JOptionPane.YES_OPTION)
		{
			recursively = true;
		}
		else
		{
			recursively = false;
		}
		
		return recursively;
	}
	
	protected void showHash(final Vector<Vector<String>> hashList)
	{
		SwingUtilities.invokeLater(new Runnable() 
		{
			//@Override
			public void run() 
			{
				new HashListWindow(frame, hashList);
			}
		});
	}

	@Override
	public void progressEvent(final ProgressEvent event) 
	{
		SwingUtilities.invokeLater(new Runnable() 
		{
			//@Override
			public void run() 
			{
				statusPanel.setStatus("File " + (currentIndex + 1) + " of " + totFiles + ": " + currentFile.getName() + " - " + event.toString());
				statusPanel.setValue(event.getPCent());
			}
		});
	}

	public void scanProgressEvent(final ScanProgressEvent event) 
	{
		SwingUtilities.invokeLater(new Runnable() 
		{
			//@Override
			public void run() 
			{
				statusPanel.setStatus(event.toString());
				statusPanel.setValue(event.getPCent());
			}
		});
	}
}
